console.log("\n\n");

// A higher order function that creates partial calls to a given lower order function
var partial = 
	function() 
	{
		// Get the name of the function that will be partially executed
		var values = [...arguments];
		var functionName = values.shift();
		
		// Return a function that calls functionName with the remaining arguments
		function partialExecution()
		{
			var newValues = [...arguments];
			return functionName.apply(this, values.concat(newValues));
		}
		return partialExecution;
	}

// An example function that does replacements in a given text
var replaceNewLines = 
	function(replacement, text)
	{
		return text.replace(/\n/g, replacement);
	}

// An example function that wraps a given text in a given tag
var wrap = 
	function(tag, text)
	{
		return "<" + tag + ">" + text + "</" + tag + ">"; 
	}

// Create several partial functions
var addBreaks = partial(replaceNewLines, "<br/>\n");
var wrapParagraph = partial(wrap, "p");
var wrapBlockquote = partial(wrap, "blockquote");

// A higher order function that creates composite calls to a given list of lower order functions
var compose = 
	function() 
	{
		var values = arguments;
		// Get the last function from the argument list
		var start = values.length - 1;
		
		// Return a function that 
		function composition()
		{
			var result = values[start].apply(this, arguments);
			start = start - 1;
			while (start >= 0)
			{
				result = values[start].call(this, result);
				start = start - 1;
			}
			return result;
		}
		return composition;
	}

// Use the composition function to compose invocations of the replace and wrap functions
var prepareForHTML = compose(wrapBlockquote, wrapParagraph, addBreaks);

// Test the composition
var lyrics = 
	"The back street dolls\n" + 
    "And the side door johnnies\n" +
    "The wide eyed boys\n" +
    "With their bags full of money";
console.log(prepareForHTML(lyrics));

console.log("\n\nPress ENTER to exit.");
process.stdin.setRawMode(true);
process.stdin.resume();
process.stdin.on('data', process.exit.bind(process, 0));
