using System;
using System.Collections.Generic;
using System.Linq;

using GraphVertex = System.Int32;

class GraphOperations
{		
	static void Main() 
	{
		Console.WriteLine("\n\n");
		
		Graph graph1 = new Graph("Graph 1", Edge(3, 4), Edge(4, 5), Edge(1, 4), Edge(1, 5), Edge(3, 5));
		Graph graph2 = new Graph("Graph 2", Edge(1, 2), Edge(2, 3), Edge(3, 4), Edge(4, 5), Edge(1, 3), Edge(1, 4), Edge(1, 5), Edge(2, 4), Edge(2, 5), Edge(3, 5));
		Graph graph3 = new Graph("Graph 3", Edge(1, 2), Edge(2, 3), Edge(3, 4), Edge(1, 3), Edge(1, 4), Edge(5, 6), Edge(5, 7));
		Graph graph4 = new Graph("Graph 4", Edge(1, 2), Edge(2, 3), Edge(1, 4), Edge(5, 5), Edge(5, 7));

		IsSubgraphOf(graph1, graph2);
		IsSubgraphOf(graph2, graph1);
		IsSupergraphOf(graph2, graph1);
		IsSubgraphOf(graph3, graph4);
		IsSubgraphOf(graph4, graph3);
		IsSubgraphOf(graph4, graph4);
		
		Console.WriteLine("\n\n");
		
		Negative(graph1);
		
		Console.WriteLine("\n\nPress ENTER to exit.");
		Console.ReadKey();
	}

	// A method that tests whether a given graph is a subgraph of another one
	static void IsSubgraphOf(Graph graph1, Graph graph2) =>
		Console.WriteLine("Is graph \"{0}\" a subgraph of \"{1}\"? Answer: {2}.\n", graph1.Name, graph2.Name, graph1.IsSubgraphOf(graph2));

	// A method that tests whether a given graph is a supergraph of another one
	static void IsSupergraphOf(Graph graph1, Graph graph2) =>
		Console.WriteLine("Is graph \"{0}\" a supergraph of \"{1}\"? Answer: {2}.\n", graph1.Name, graph2.Name, graph1.IsSupergraphOf(graph2));

	// A method that tests the result of the negative operation
	static void Negative(Graph graph) =>
		Console.WriteLine("The negative of graph \"{0}\" is:\n{1}.\n", graph.Name, graph.Negative);

	class GraphEdge : Tuple<GraphVertex, GraphVertex>
	{
		public GraphEdge(GraphVertex fromVertex, GraphVertex toVertex) : base(fromVertex, toVertex)	{}
		
		public GraphVertex From => Item1;
		public GraphVertex To => Item2;
	}
	
	// Syntactic sugar meant to ease the declaration of edges
	static GraphEdge Edge(GraphVertex inVertex, GraphVertex outVertex)
	{
		return new GraphEdge(inVertex, outVertex);
	}
	
	class Graph
	{
		public Graph(string name, params GraphEdge[] edges)
		{
			Name = name;
			foreach(var edge in edges)	AddEdge(edge);
		}
		
		private void AddEdge(GraphEdge edge)
		{
			vertices.Add(edge.From);
			vertices.Add(edge.To);
			edges.Add(edge);
		}
		
		public string Name {	get;	set;	}
		private HashSet<GraphEdge> edges = new HashSet<GraphEdge>();
		private HashSet<GraphVertex> vertices = new HashSet<GraphVertex>();
		
		public override string ToString()
		{
			return string.Format("Graph \"{0}\":\n\tvertices: {1}\n\tedges: {2}", Name, string.Join(",", vertices), string.Join(",", edges));
		}
		
		// Verifies whether the current graph is a subgraph of the given one
		public bool IsSubgraphOf(Graph graph) => 
			vertices.IsSubsetOf(graph.vertices) && edges.IsSubsetOf(graph.edges);
		
		// Verifies whether the current graph is a supergraph of the given one
		public bool IsSupergraphOf(Graph graph) => 
			//vertices.IsSupersetOf(graph.vertices) && edges.IsSupersetOf(graph.edges);	// Longer solution
			graph.IsSubgraphOf(this);	// Shorter solution

		// Creates the total graph induced by the current one
		public Graph Total
		{
			get
			{
				Graph total = new Graph(Name + " (total)");
				total.vertices = vertices;
				total.edges = 
					new HashSet<GraphEdge>(
						from vertex1 in vertices
						from vertex2 in vertices
						select Edge(vertex1, vertex2));
				return total;
			}
		}	
		
		// Creates the negative graph of the current one
		public Graph Negative
		{
			get
			{
				Graph negative = Total;
				negative.Name = Name + " (negative)";
				negative.edges.ExceptWith(edges);
				return negative;
			}
		}
	}
}
