import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.Iterable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.OptionalInt;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

class HillClimbingTraversal
{  
	public static void main(String args[])
	{
		System.out.println("\n\n");

		Graph graph = new Graph(Edge(25, 100), Edge(25, 40), Edge(40, 150), Edge(15, 32), Edge(32, 100), Edge(15, 60), Edge(60, 150));

		TestHillClimbing(graph, 25);
		
		System.out.println("\n\n");
		
		TestHillClimbing(graph, 15);
		
		System.out.println("\n\nPress ENTER to exit.");
		try
		{
			BufferedReader systemInReader = new BufferedReader(new InputStreamReader(System.in));
			int enter = systemInReader.read();
		}
		catch(IOException exception)	{}
	}
	
	// A method that invokes the Hill climbing traversal from a given vertex
	static void TestHillClimbing(Graph graph, Integer startVertex)
	{
		System.out.println("A path from " +startVertex + " to a (local) maximum is:");
		graph.HillClimbingTraversal(startVertex).forEach(vertex -> System.out.println("\tThe path passes through:" + vertex));		
	}
	
	static class GraphEdge
	{
		public GraphEdge(Integer fromVertex, Integer toVertex)
		{
			From = fromVertex;
			To = toVertex;
		}
		
		public final Integer From;
		public final Integer To;
	}
	
	// Syntactic sugar meant to ease the declaration of edges
	static GraphEdge Edge(Integer inVertex, Integer outVertex)
	{
		return new GraphEdge(inVertex, outVertex);
	}

	static class Graph
	{
		public Graph(GraphEdge... edges)
		{
			for(GraphEdge edge : edges)	AddEdge(edge);
		}
		
		private void AddEdge(GraphEdge edge)
		{
			AddVertex(edge.From);
			AddVertex(edge.To);
			outgoingEdges.get(edge.From).add(edge);
		}
		
		private void AddVertex(Integer vertex)
		{
			vertices.add(vertex);
			if(!outgoingEdges.containsKey(vertex))	outgoingEdges.put(vertex, new ArrayList<GraphEdge>());			
		}
		
		private HashSet<Integer> vertices = new HashSet<Integer>();
		private Hashtable<Integer, ArrayList<GraphEdge>> outgoingEdges = new Hashtable<Integer, ArrayList<GraphEdge>>(); // In most cases HashSet would be preferable to ArrayList
		
		// Iterates all vertices in a path starting from a given vertex and ending in a (local) maximum. Uses the Hill climbing heuristics.
		public Stream<Integer> HillClimbingTraversal(Integer start)
		{
			Iterable<Integer> HillClimbingIterable = 
				new Iterable<Integer>()
				{
					public Iterator<Integer> iterator()
					{		
						return
							new Iterator<Integer>()
							{
								private OptionalInt previous = OptionalInt.empty();
								private OptionalInt current = OptionalInt.of(start);
								
								public boolean hasNext()
								{
									return !previous.isPresent() || current.isPresent() && previous.getAsInt() < current.getAsInt();									
								}

								public Integer next()
								{
									previous = current;
									current = outgoingEdges.get(current.getAsInt()).stream().map(edge -> edge.To).mapToInt(Integer::intValue).max();
									return previous.getAsInt();
								}
								
								public void remove()	{	throw new UnsupportedOperationException();	}
							};
					}
				};

			return StreamSupport.stream(HillClimbingIterable.spliterator(), false);
		}					
	}		
} 
