import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.function.Function;
import java.util.stream.Stream;

class LineSpacing
{  
	public static void main(String args[])
	{
		System.out.println("\n\n");

		// Declare several pages of text. Each page is a list of its consecutive lines. 
		Page page1 = 
			new Page(
				new LineData("TITLE:", 20), 
				new LineData("EXAMPLE DOCUMENT", 18), 
				new LineData("authot:", 15), 
				new LineData("Vladislav Nenchev", 13), 
				new LineData("Faculty of Mathematics and Informatics", 12), 
				new LineData("Sofia University", 11), 
				new LineData("page 1", 1));
		Page page2 = 
			new Page(
				new LineData("This page contains very little text.", 22), 
				new LineData("It is to be filled at a later stage.", 21), 
				new LineData("page 2", 1));
		Page page3 = 
			new Page(
				new LineData("COLCLUSION", 22), 
				new LineData("The aim of this example document is to serve ", 20), 
				new LineData("as test data for the most common spacing distance ", 19), 
				new LineData("between consecutive lines in a given document.", 18), 
				new LineData("page 3", 1));
		
		// Form a document - an collection of the pages
		ArrayList<Page> document = new ArrayList<>(Arrays.asList(page1, page2, page3));
		
		// Count the number of occurrences for each distance
		Map<Integer, Long> lineSpacingCounts = 
			document.stream().flatMap(page -> page.LineSpacingData().stream()).
				collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));

		// Get the most common line spacing
		int mostCommonLineSpacing = 
			lineSpacingCounts.keySet().stream().
				reduce((key1, key2) -> lineSpacingCounts.get(key1) > lineSpacingCounts.get(key2) ? key1 : key2).
					orElse(-1);
		
		System.out.println("The most common line spacing in this document is:\n" + mostCommonLineSpacing);
		
		System.out.println("\n\nPress ENTER to exit.");
		try
		{
			BufferedReader systemInReader = new BufferedReader(new InputStreamReader(System.in));
			int enter = systemInReader.read();
		}
		catch(IOException exception)	{}
	}
} 

// A class that represents a page in a document
class Page
{
	public Page(LineData... lines)
	{
		data = Stream.of(lines).collect(Collectors.toCollection(ArrayList::new));
	}
	
	// Collects all spacing distances between consecutive lines in this page
	ArrayList<Integer> LineSpacingData()
	{
		return 
			IntStream.range(0, data.size() - 1).
				boxed().
					map(index -> data.get(index).LineHeight - data.get(index + 1).LineHeight).
						collect(Collectors.toCollection(ArrayList::new));
	}
	
	ArrayList<LineData> data;		// A collection of all consecutive lines in this page
}

// A class that represents a line of text positioned within a page
class LineData
{
	public LineData(String lineText, int lineHeight)
	{
		LineText = lineText;
		LineHeight = lineHeight;
	}
	
	public String LineText;	// The text contents of the current line
	public int LineHeight;		// The height position of the line within its containing page (the Y coordinate of the line's baseline)
}
