import math
import msvcrt

print("\n\n")

# Determines whether the given number is prime. The verification is done in procedural (old) style.
def IsPrimeProcedural(number):
	if number < 2: return False
	for lesser in range(2, math.floor(math.sqrt(number)) + 1):
		if number % lesser == 0: return False
	return True

# Calculates the next prime number equal or greater to the given value
def GetNextPrime(start):
	while not IsPrimeProcedural(start):
		start += 1
	return start	

# Verifies whether a given number is a sum of the cubes of three prime numbers
def VerifySumOf3Cubes(prime):
	for lesser1 in range(math.floor(math.log(prime, 3)) + 1):
		if IsPrimeProcedural(lesser1):
			for lesser2 in range(math.floor(math.log(prime, 3)) + 1):
				if IsPrimeProcedural(lesser2):
					for lesser3 in range(math.floor(math.log(prime, 3)) + 1):
						if IsPrimeProcedural(lesser3):
							if prime == math.pow(lesser1, 3) + math.pow(lesser2, 3) + math.pow(lesser3, 3): return True
	return False
	
# Finds the minimal prime number that is a sum of the cubes of three prime numbers. The calculation is done in procedural (old) style.
minimalSumOf3Cubes = 2
while not VerifySumOf3Cubes(minimalSumOf3Cubes):
	minimalSumOf3Cubes = GetNextPrime(minimalSumOf3Cubes + 1)
print("The minimal prime number that is sum of the cubes of three prime numbers is:", minimalSumOf3Cubes)

print("\n\n")

# An infinite generator of natural numbers
def NaturalNumbers():
	number = 0
	while True:
		yield number
		number += 1

# Determines whether the given number is prime. The verification is done in declarative (new) style.
def IsPrimeDeclarative(number):
	return number > 1 and all(not number % lesser == 0 for lesser in range(2, math.floor(math.sqrt(number)) + 1))

# An infinite generator of prime numbers
def PrimeNumbers():
	return (number for number in NaturalNumbers() if IsPrimeDeclarative(number))
	
# Syntactical sugar meant to ease the retrieval of the first value in an iteration that matches a given condition
def First(iterator, predicate):
	return next(element for element in iterator() if predicate(element))

# Finds the minimal prime number that is a sum of the cubes of three prime numbers. The calculation is done in declarative (new) style.
minimalSumOf3Cubes = \
	First(PrimeNumbers, \
		lambda prime: \
			any(prime == math.pow(lesser1, 3) + math.pow(lesser2, 3) + math.pow(lesser3, 3) \
			for lesser1 in range(math.floor(math.log(prime, 3)) + 1) if IsPrimeDeclarative(lesser1) \
			for lesser2 in range(math.floor(math.log(prime, 3)) + 1) if IsPrimeDeclarative(lesser2) \
			for lesser3 in range(math.floor(math.log(prime, 3)) + 1) if IsPrimeDeclarative(lesser3)))
print("The minimal prime number that is sum of the cubes of three prime numbers is:", minimalSumOf3Cubes)

# A more readable realization of string reversal
def Reverse(string):
	return string[::-1]
	
# Finds the minimal palindromic prime number with at least four digits. The calculation is done in declarative (new) style.
minimal4DigitPalindrome = First(PrimeNumbers, lambda prime: prime > 999 and Reverse(str(prime)) == str(prime))
print("The minimal prime number that is palindrome with at least four digits is:", minimal4DigitPalindrome)

print("\n\nPress ENTER to exit.")
msvcrt.getch()
