import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.StringBuilder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.concurrent.Callable;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

class Pipelines
{  
	public static void main(String args[])
	{
		System.out.println("\n\n");

		ArrayList<Party> parties = new ArrayList<Party>();
		parties.add(new Person(1, "Graham Chapman", "An English comedian, a founding member of the Pythons."));
		parties.add(new Person(2, "John Cleese", "An English comedian, a founding member of the Pythons."));
		parties.add(new Person(3, "Terry Gilliam", "An American animator, a founding member of Monty Python."));
		parties.add(new Person(4, "Eric Idle", "An English comedian, a founding member of Monty Python."));
		parties.add(new Person(5, "Terry Jones", "A Welsh comedian, a founding member of Monty Python."));
		parties.add(new Person(6, "Michael Palin", "An English comedian, a founding member of Monty Python."));
		parties.add(new Person(7, "Carol Cleveland", "An English comedian, an associate member of Monty Python."));
		parties.add(new Company(8, "British Broadcasting Corporation", "A British television company."));
		parties.add(new Company(9, "HandMade Films", "A British film production company."));
		
		// Find all valid parties by the given ids and obtain a final integrated text from their sorted display texts
		String ids = "1,2,3,4,7,8,9";
		String finalText = 
			Stream.of(ids.split(",")).map(part -> Integer.parseInt(part)).map(id -> Party.GetPartyById(parties, id)).
				filter(party -> party.IsValid()).sorted(new PartyComparator()).map(person -> person.GetDisplayText()).collect(Collectors.joining("\n"));
		System.out.println("The final integrated text is:\n" + finalText);

		System.out.println("\n\n");
		
		// Process specifically the data for each party that corresponds to an id from the given list
		ids = "2,8,7,9,4";
		Stream.of(ids.split(",")).map(part -> Integer.parseInt(part)).map(id -> Party.GetPartyById(parties, id)).forEach(party -> party.PipelineProcess());
		
		System.out.println("\n\nPress ENTER to exit.");
		try
		{
			BufferedReader systemInReader = new BufferedReader(new InputStreamReader(System.in));
			int enter = systemInReader.read();
		}
		catch(IOException exception)	{}
	}
} 

enum PartyType	{	Person, Company	}

abstract class Party
{
	public Party(Integer id, String name, String description)
	{
		partyId = id;
		partyName = name;
		partyDescription = description;
	}
	
	// Getter and setter methods
	public Integer PartyId() {	return partyId;	}
	abstract public PartyType Type();
	public String PartyName()	{	return partyName;	}
	public void PartyName(String name)	{	partyName = name;	}
	public String PartyDescription()	{	return partyDescription;	}
	public void PartyDescription(String description)	{	partyDescription = description;	}
	
	// Finds a party in a given list of parties by the given id
	public static Party GetPartyById(List<Party> parties, int id)
	{
		return parties.stream().filter(party -> party.PartyId() == id).findFirst().get();
	}
	
	// Get the display text for the current party. DisplayText = PartyName + dash + PartyDescription.
	public String GetDisplayText()	{	return PartyName() + " - " + PartyDescription();	}
	
	// Verifies the validity of the data in the current party
	abstract public boolean IsValid();
	
	// Process the data of the current party in a specific way
	abstract public void PipelineProcess();
	
	private Integer partyId;
	private String partyName;
	private String partyDescription;
}

class PartyComparator implements Comparator<Party>
{
	public int compare(Party party1, Party party2)
	{
		return  party1.PartyName().compareTo(party2.PartyName());
	}
}
	
class Person extends Party
{
	public Person(Integer id, String name, String description)	{	super(id, name, description);	}
	public PartyType Type()	{	return PartyType.Person;	}
	public boolean IsValid()	{	return PartyDescription().contains("Monty Python");	}
	public void PipelineProcess()	{	System.out.println(PartyName().toUpperCase());	}
}

class Company extends Party
{
	public Company(Integer id, String name, String description)	{	super(id, name, description);	}
	public PartyType Type()	{	return PartyType.Company;	}
	public boolean IsValid()	{	return PartyName().endsWith(" Corporation") || PartyName().endsWith(" Ltd.");	}
	public void PipelineProcess()	{	System.out.println(PartyDescription().split("\\W+").length);	}
}
