import msvcrt
import time

from itertools import *
from math import *

print("\n\n")

# An infinite generator of prime numbers using the sieve of Eratosthenes with lambda expressions
def EratosthenesLambda():
	generated = []
	for number in count(2):
		if all(map(lambda lesser: number % lesser > 0, generated)):
			generated.append(number)
			yield number

# An infinite generator of prime numbers using the sieve of Eratosthenes without lambda expressions
def EratosthenesNonLambda():
	generated = []
	for number in count(2):
		isPrime = True
		for lesser in generated:
			if number % lesser == 0:
				isPrime = False
				break
		if isPrime: 
			generated.append(number)
			yield number
		
# A method that measures the time needed for e given generator to produce a certain amount of elements
def MeasureGenerator(generator, limit, name):
	startTime = time.time()
	for prime in islice(generator(), limit): pass # Just generate the prime without doing anything
	print("{:>26} method: {:<}".format(name, time.time() - startTime))

# A method that invokes the measurements for all generators of prime numbers
def MeasureAllGenerators(limit):
	print("Tests for generation of first", limit, "prime numbers (time is measured in seconds):")
	MeasureGenerator(EratosthenesLambda, limit, "Lambda Eratosthenes")
	MeasureGenerator(EratosthenesNonLambda, limit, "Non-lambda Eratosthenes")
	print("\n")

MeasureAllGenerators(50)
MeasureAllGenerators(100)
MeasureAllGenerators(500)
MeasureAllGenerators(1000)
MeasureAllGenerators(5000)
MeasureAllGenerators(10000)
MeasureAllGenerators(20000)
MeasureAllGenerators(30000)
MeasureAllGenerators(40000)
MeasureAllGenerators(50000)

# A method that invokes the measurements for the non-lambda generator, but with different methods (with or without lambda) for filtering of the result
def MeasureNonLambdaGenerator(limit):
	print("Tests for generation and then filter of first", limit, "prime numbers (time is measured in seconds):")
	startTime = time.time()
	for prime in islice(EratosthenesNonLambda(), limit): 
		if prime % 100 < 50: 
			pass # Just generate the prime without doing anything
	print("{:>26} method: {:<}".format("Lambda filter", time.time() - startTime))
	startTime = time.time()
	for prime in filter(lambda number: number % 100 < 50, islice(EratosthenesNonLambda(), limit)): 
		pass # Just generate the prime without doing anything
	print("{:>26} method: {:<}".format("Non-lambda filter", time.time() - startTime))
	print("\n")

MeasureNonLambdaGenerator(30)
MeasureNonLambdaGenerator(50)
MeasureNonLambdaGenerator(100)
MeasureNonLambdaGenerator(200)
MeasureNonLambdaGenerator(300)
MeasureNonLambdaGenerator(400)
MeasureNonLambdaGenerator(500)
MeasureNonLambdaGenerator(1000)
MeasureNonLambdaGenerator(2000)
MeasureNonLambdaGenerator(3000)
MeasureNonLambdaGenerator(4000)
MeasureNonLambdaGenerator(5000)
MeasureNonLambdaGenerator(10000)
MeasureNonLambdaGenerator(20000)
MeasureNonLambdaGenerator(30000)
MeasureNonLambdaGenerator(40000)
MeasureNonLambdaGenerator(50000)

print("\n\nPress ENTER to exit.")
msvcrt.getch()
