import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.StringBuilder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.concurrent.Callable;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

class ReplaceIdsWithTexts
{  
	public static void main(String args[])
	{
		System.out.println("\n\n");

		ArrayList<Party> parties = new ArrayList<Party>();
		parties.add(new Person(1, "Graham Chapman", "An English comedian, a founding member of Monty Python."));
		parties.add(new Person(2, "John Cleese", "An English comedian, a founding member of Monty Python."));
		parties.add(new Person(3, "Terry Gilliam", "An American animator, a founding member of Monty Python."));
		parties.add(new Person(4, "Eric Idle", "An English comedian, a founding member of Monty Python."));
		parties.add(new Person(5, "Terry Jones", "A Welsh comedian, a founding member of Monty Python."));
		parties.add(new Person(6, "Michael Palin", "An English comedian, a founding member of Monty Python."));
		parties.add(new Person(7, "Carol Cleveland", "An English comedian, an associate member of Monty Python."));
		parties.add(new Company(8, "BBC", "A British television company."));
		parties.add(new Company(9, "HandMade Films", "A British film production company."));
		
		ReplaceIdsWithTextsTestProcedural("3,4,9,6,5,8", parties);
		ReplaceIdsWithTextsTestPipeline("3,4,9,6,5,8", parties);
		
		System.out.println("\n\n");
		
		ReplaceIdsWithTextsTestProcedural("8,7,2,1", parties);
		ReplaceIdsWithTextsTestPipeline("8,7,2,1", parties);

		System.out.println("\n\nPress ENTER to exit.");
		try
		{
			BufferedReader systemInReader = new BufferedReader(new InputStreamReader(System.in));
			int enter = systemInReader.read();
		}
		catch(IOException exception)	{}
	}
	
	// A method that invokes the replacement for a given list of ids and prints the result. Uses the procedural (old) style transformations.
	static void ReplaceIdsWithTextsTestProcedural(String ids, ArrayList<Party> parties)transformations
	{
		System.out.printf("The result for ids %s is:\n%s\n\n", ids, ReplaceIdsWithTextsProcedural(ids, parties));
	}
		
	// A method that invokes the replacement for a given list of ids and prints the result. Uses the pipeline (new) style transformations.
	static void ReplaceIdsWithTextsTestPipeline(String ids, ArrayList<Party> parties)
	{
		System.out.printf("The result for ids %s is:\n%s\n\n", ids, ReplaceIdsWithTextsPipeline(ids, parties));
	}
	
	// A methods that transforms a list of ids to a sorted concatenation of all persons referred by the ids. The transformations are done in procedural (old) style.
	static String ReplaceIdsWithTextsProcedural(String ids, ArrayList<Party> parties)
	{
		ArrayList<Person> persons = new ArrayList<Person>();
		String[] parts = ids.split(",");
		for(String part : parts)
		{
			Integer id = Integer.parseInt(part);
			for(Party party : parties)
			{
				if(party.PartyId() == id && party.Type() == PartyType.Person)
				{
					persons.add((Person) party);
					break;
				}
			}
        }
		
		Collections.sort(persons, new PartyComparator());
		
		StringBuilder stringBuilder = new StringBuilder();
		boolean first = true;
		for(Person person : persons)
		{
			if(!first)	stringBuilder.append("\n");
			stringBuilder.append(person.PartyName() + " - " + person.PartyDescription());
			first = false;
		}
		
		return stringBuilder.toString();
	}
	
	// A methods that transforms a list of ids to a sorted concatenation of all persons referred by the ids. The transformations are done in pipeline (new) style.
	static String ReplaceIdsWithTextsPipeline(String ids, ArrayList<Party> parties)
	{
		return 
			Stream.of(ids.split(",")).map(part -> Integer.parseInt(part)).map(id -> parties.stream().filter(party -> party.PartyId() == id).findFirst().get()).
				filter(party -> party.Type() == PartyType.Person).sorted(new PartyComparator()).map(person -> person.PartyName() + " - " + person.PartyDescription()).collect(Collectors.joining("\n"));
	}
} 

enum PartyType	{	Person, Company	}

abstract class Party
{
	public Party(Integer id, String name, String description)
	{
		partyId = id;
		partyName = name;
		partyDescription = description;
	}
	
	// Getter and setter methods
	public Integer PartyId() {	return partyId;	}
	abstract public PartyType Type();
	public String PartyName()	{	return partyName;	}
	public void PartyName(String name)	{	partyName = name;	}
	public String PartyDescription()	{	return partyDescription;	}
	public void PartyDescription(String description)	{	partyDescription = description;	}
	
	private Integer partyId;
	private String partyName;
	private String partyDescription;
}

class PartyComparator implements Comparator<Party>
{
	public int compare(Party party1, Party party2)
	{
		return  party1.PartyName().compareTo(party2.PartyName());
	}
}
	
class Person extends Party
{
	public Person(Integer id, String name, String description)	{	super(id, name, description);	}
	public PartyType Type()	{	return PartyType.Person;	}
}

class Company extends Party
{
	public Company(Integer id, String name, String description)	{	super(id, name, description);	}
	public PartyType Type()	{	return PartyType.Company;	}
}
