import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.Set;
import java.util.stream.Stream;
import java.util.stream.Collectors;

class SerialGraphs
{  
	public static void main(String args[])
	{
		System.out.println("\n\n");

		Graph graph1 = new Graph(Edge(1, 2), Edge(1, 4), Edge(2, 3), Edge(4, 3), Edge(3, 5), Edge(5, 4));		
		System.out.println("Is the first graph forward serial? Answer: " + graph1.IsForwardSerial());
		System.out.println("Is the first graph backward serial? Answer: " + graph1.IsBackwardSerial());
		
		System.out.println("\n\n");
				
		Graph graph2 = new Graph(Edge(1, 1), Edge(1, 2), Edge(1, 3), Edge(1, 4), Edge(3, 5));		
		System.out.println("Is the second graph forward serial? Answer: " + graph2.IsForwardSerial());
		System.out.println("Is the second graph backward serial? Answer: " + graph2.IsBackwardSerial());

		System.out.println("\n\nPress ENTER to exit.");
		try
		{
			BufferedReader systemInReader = new BufferedReader(new InputStreamReader(System.in));
			int enter = systemInReader.read();
		}
		catch(IOException exception)	{}
	}
	
	static class GraphEdge
	{
		public GraphEdge(Integer fromVertex, Integer toVertex)
		{
			From = fromVertex;
			To = toVertex;
		}
		
		public final Integer From;
		public final Integer To;
	}
	
	// Syntactic sugar meant to ease the declaration of edges
	static GraphEdge Edge(Integer inVertex, Integer outVertex)
	{
		return new GraphEdge(inVertex, outVertex);
	}

	static class Graph
	{
		public Graph(GraphEdge... graphEdges)
		{
			edges = Stream.of(graphEdges).collect(Collectors.toSet());
			vertices = edges.stream().map(edge -> edge.From).collect(Collectors.toSet());
			vertices.addAll(edges.stream().map(edge -> edge.To).collect(Collectors.toSet()));
		}
		
		// Checks whether the current graph is forward serial
		public boolean IsForwardSerial()
		{
			return vertices.stream().allMatch(vertex -> edges.stream().anyMatch(edge -> edge.From == vertex));
		}
		
		// Checks whether the current graph is backward serial
		public boolean IsBackwardSerial()
		{
			return vertices.stream().allMatch(vertex -> edges.stream().anyMatch(edge -> edge.To == vertex));
		}
		
		private Set<Integer> vertices;
		private Set<GraphEdge> edges;
	}		
} 
