using System;
using System.Text.RegularExpressions;

class StringValidations
{
	static void Main() 
	{
		Console.WriteLine("\n\n");
		
		Validate(ValidationType.Number, "00564", ValidateNumberProcedural);
		Validate(ValidationType.Number, "5473737", ValidateNumberProcedural);
		Validate(ValidationType.Number, "0", ValidateNumberProcedural);
		Validate(ValidationType.Number, "3424g65", ValidateNumberProcedural);
		
		Console.WriteLine("\n\n");
		
		Validate(ValidationType.Number, "00564", ValidateNumberRegex);
		Validate(ValidationType.Number, "5473737", ValidateNumberRegex);
		Validate(ValidationType.Number, "0", ValidateNumberRegex);
		Validate(ValidationType.Number, "3424g65", ValidateNumberRegex);

		Console.WriteLine("\n\n");
		
		Validate(ValidationType.Email, "test@example.com", ValidateEmailProcedural);
		Validate(ValidationType.Email, "test.rr34343r@example.com", ValidateEmailProcedural);
		Validate(ValidationType.Email, "test.rr34343r.tgf_d.wdfs-gsgd-gsg.gfgfd.gfg@example.dfdf.com", ValidateEmailProcedural);
		Validate(ValidationType.Email, "test@example", ValidateEmailProcedural);
		Validate(ValidationType.Email, "ffg@test@example.com", ValidateEmailProcedural);
		Validate(ValidationType.Email, "@example.com", ValidateEmailProcedural);
		Validate(ValidationType.Email, "example.com", ValidateEmailProcedural);
		Validate(ValidationType.Email, "test@example..com", ValidateEmailProcedural);
		Validate(ValidationType.Email, ".test@example.com", ValidateEmailProcedural);
		
		Console.WriteLine("\n\n");
		
		Validate(ValidationType.Email, "test@example.com", ValidateEmailRegex);
		Validate(ValidationType.Email, "test.rr34343r@example.com", ValidateEmailRegex);
		Validate(ValidationType.Email, "test.rr34343r.tgf_d.wdfs-gsgd-gsg.gfgfd.gfg@example.dfdf.com", ValidateEmailRegex);
		Validate(ValidationType.Email, "test@example", ValidateEmailRegex);
		Validate(ValidationType.Email, "ffg@test@example.com", ValidateEmailRegex);
		Validate(ValidationType.Email, "@example.com", ValidateEmailRegex);
		Validate(ValidationType.Email, "example.com", ValidateEmailRegex);
		Validate(ValidationType.Email, "test@example..com", ValidateEmailRegex);
		Validate(ValidationType.Email, ".test@example.com", ValidateEmailRegex);
		
		Console.WriteLine("\n\nPress ENTER to exit.");
		Console.ReadKey();
	}
	
	// Type of the validation - whether the input is validated as natural number or as an e-mail
	enum ValidationType {	Number, Email	};
	
	// Generic method used to invoke the different validations
	static void Validate(ValidationType validationType, string input, Predicate<string> validator) =>
		Console.WriteLine("Is \"{0}\" a valid {1}? Answer: {2}.\n", input, validationType.ToString().ToLower(), validator(input));
	
	// Validate whether the input is a natural number. The validation is done in procedural (old) style.
	static bool ValidateNumberProcedural(string input)
	{
		if(input.Length == 0)	return false;
		if(input[0] == '0' && input.Length > 1)	return false;

		for(int i =0; i < input.Length; i++)
		{
			if(((int) input[i] < 48) || ((int) input[i] > 57))	return false;
		}
		
		return true;
	}
	
	// Validate whether the input is a natural number. The validation is done in regular expression (new) style.
	static bool ValidateNumberRegex(string input) => Regex.IsMatch(input, @"^0$|^[1-9][0-9]*$");
	
	// Checks if the given symbol is invalid for an e-mail
	static bool CheckSymbol(char symbol)
	{
		int code = (int) symbol;
		return 
			(code < 65 || code > 90) && (code < 97 || code > 122) && (code < 48 || code > 57) && 
			symbol != '-' && symbol != '_';
	}
	
	// Checks if any of the given tokens is invalid for an e-mail
	static bool CheckTokens(String[] tokens, int minimalCount = 1)
	{
		if(tokens.Length < minimalCount) return true;
			
		foreach(var token in tokens)
		{
			if(token.Length == 0) return true;
			for(int i =0; i < token.Length; i++)
				if(CheckSymbol(token[i])) return true;
		}
		
		return false;
	}	

	// Validate whether the input is a e-mail. The validation is done in procedural (old) style.
	static bool ValidateEmailProcedural(string input)
	{
		if(input.Length == 0)	return false;
		
		String[] parts = input.Split('@');
		if(parts.Length != 2)	return false;
		
		if(CheckTokens(parts[0].Split('.'))) return false;
		if(CheckTokens(parts[1].Split('.'), 2)) return false;
		
		return true;
	}
	
	// Validate whether the input is a e-mail. The validation is done in regular expression (new) style.
	static bool ValidateEmailRegex(string input) => Regex.IsMatch(input, @"^[_\-\w]+(\.[_\-\w]+)*@[_\-\w]+(\.[_\-\w]+)+$");
}
