using System;
using System.Collections.Generic;
using System.Linq;

using NodeValue = System.Int32;

class TreeEnumerators
{
	static void Main() 
	{
		Console.WriteLine("\n\n");
				
		TreeNode tree = 
			Tree(12, 
				Tree(7, 
					Tree(42), 
					Tree(3)), 
				Tree(19, 
					Tree(44, 
						Tree(100), 
						Tree(32), 
						Tree(0, 
							Tree(13)))), 
				Tree(142));

		Console.WriteLine("This is a pre-order traversal of the tree:");
		tree.PreTraverse().ForEach(
			node => Console.WriteLine(string.Format("The current node is at height {0} and has value {1}.", node.Height, node.Value)));
		
		Console.WriteLine("\n\n");

		Console.WriteLine("This is a post-order traversal of the tree:");
		tree.PostTraverse().ForEach(
			node => Console.WriteLine(string.Format("The current node is at height {0} and has value {1}.", node.Height, node.Value)));
				
		Console.WriteLine("\n\nPress ENTER to exit.");
		Console.ReadKey();
	}
	
	class TreeNode
	{
		public TreeNode(NodeValue value, params TreeNode[] childrenNodes)
		{
			Value = value;
			children.AddRange(childrenNodes);
			Height = children.Select(child => child.Height + 1).DefaultIfEmpty().Max();
		}
		
		// Traverses the tree with the current node as a root in a pre-order
		public IEnumerable<TreeNode> PreTraverse()
		{
			yield return this;

			foreach(var immediateChild in children) 
				foreach(var child in immediateChild.PreTraverse())
					yield return child;
		}
		
		// Traverses the tree with the current node as a root in a post-order
		public IEnumerable<TreeNode> PostTraverse()
		{
			foreach(var immediateChild in children) 
				foreach(var child in immediateChild.PostTraverse())
					yield return child;

			yield return this;
		}
		
		public NodeValue Value	{	get;	}
		public int Height	{	get;	}
		private List<TreeNode> children = new List<TreeNode>();
	}
	
	// Syntactic sugar meant to ease the declaration of trees
	static TreeNode Tree(NodeValue value, params TreeNode[] childrenNodes)
	{
		return new TreeNode(value, childrenNodes);
	}	
}

// Extending the IEnumerable<T> interface with a ForEach method (like the one from List<T>)
public static class IEnumerableExtensions
{
	public static void ForEach<T>(this IEnumerable<T> iEnumerable, Action<T> action)
	{
		foreach(T item in iEnumerable)	action(item);
	}
}
