import msvcrt
import time

from itertools import *

print("\n\n")

# Generator of subsets
def subsets(set_):
	yield from chain.from_iterable(combinations(set_, size) for size in range(len(set_) + 1))
	
#  A "naive" (direct) solution for the zero sum subset problem
def hasZeroSumSubsetNaive(set_):
	return any(map(lambda subset: len(subset) > 0 and sum(subset) == 0, subsets(set_)))

# A pseudo-polynomial solution (using dynamic programming) of the zero sum subset problem
def hasZeroSumSubsetDynamic(set_):
	A = sum(filter(lambda n: n < 0, set_))
	B = sum(filter(lambda n: n > 0, set_))
	return Q(list(set_), len(set_) - 1, A, B)[0]

# Calculate the auxiliary Q array for the dynamic programming solution of the zero sum subset problem
def Q(set_, N, A, B):
	if N == 0:
		return dict(map(lambda number: (number, number == set_[N]), range(A, B + 1)))
	else:
		previousQ = Q(set_, N - 1, A, B)
		return dict(map(lambda number: (number, previousQ[number] or number == set_[N] or (A <= number - set_[N] and number - set_[N] <= B and previousQ[number - set_[N]])), range(A, B + 1)))

# A method that measures the time needed for e given solution to solve the problem for a certain set
def testSolution(solution, set_, solutionName):
	startTime = time.time()	
	result = solution(set_)
	print("{:>8} solution runs for {:<25} Answer: {}".format(solutionName, time.time() - startTime, result))

# A method that invokes the measurements for all solutions of the problem
def test(set_):
	print("Testing solutions for set {} (time is measured in seconds):".format(set_))
	testSolution(hasZeroSumSubsetNaive, set_, "Naive")
	testSolution(hasZeroSumSubsetDynamic, set_, "Dynamic")
	print("\n")

test(set(list(range(1, 21))))	
test(set(list(range(1, 22))))
test(set(list(range(1, 23))))
test(set(list(range(1, 24))))
test(set(list(range(1, 25))))
test(set(list(range(1, 26))))
test(set(list(range(1, 27))))

test([34, 453, 2, 65, 67459951, 34, 77, 33, 5, 76, 22, 5, 77, 336, 654, 1, 43, 65, -65])

test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, -190])
test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, -210])
test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, -231])
test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, -253])
test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, -276])
test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, -300])
test([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -325])


print("\n\nPress ENTER to exit.")
msvcrt.getch()
